# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-28 17:51
from __future__ import unicode_literals

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0049_auto_20160428_1750'),
    ]

    operations = [
        migrations.AlterField(
            model_name='staffingrequest',
            name='communication_method',
            field=models.IntegerField(choices=[(0, 'slack'), (1, 'email')]),
        ),
    ]
