@import 'orchestra/static/orchestra/common/css/common.scss';
@import 'orchestra/static/orchestra/common/css/base';

$task-card-height: 8.5rem;
$task-card-padding: 1rem;
$task-card-margin-bottom: 1.5rem;

// TODO(jrbotros): use variable from bootstrap-sass?
$bootstrap-md: "(min-width: 992px)";

// TODO(marcua): Move this to tasktable.scss
.tasks-section {
  .section-header {
    .warning-message {
      font-style: italic;
      margin-left: 1.5rem;
      font-size: 1.5rem;
    }

    // New task buttons
    .btn-new-task {
      width: 150px;
      margin: 0 5px;
    }
    .btn-new-entry-task {
      @extend .btn-new-task;
      background-color: $dark-green;
      &.disabled {
        background-color: rgba($dark-green, 0.6);
      }
    }
    .btn-new-review-task {
      @extend .btn-new-task;
      background-color: $light-green;
      &.disabled {
        background-color: rgba($light-green, 0.6);
      }
    }
  }
  tr.task-row {
    cursor: pointer;
  }
  .table {
    margin-bottom: 0;
    font-size: 1.4rem;
  }
  .row {
    overflow-x: scroll;
  }
  .st-sort-ascent:after{
    content: '\25B2';
  }
  .st-sort-descent:after{
    content: '\25BC';
  }
  .tasks-search-bar-container {
    display: flex;
    justify-content: flex-end;
  }
  .tasks-search-bar {
    margin: 5px 10px;
    max-width: 150px;
    background: white;
    border: $light-gray 1px solid;
    color: $dark-gray;
    display: flex;
    justify-content: flex-start;
    -webkit-box-shadow: inset 0 1px 1px rgba(0,0,0,.075);
    box-shadow: inset 0 1px 1px rgba(0,0,0,.075);
    width: 100%;
    height: 34px;
    font-size: 14px;
    line-height: 1.42857143;
    border-radius: 3px;
    i {
      padding: 8px;
      color: $light-gray;
    }
    input {
      height: 100%;
      width: calc(100% - 30px);
      padding: 8px 2px 10px 2px;
      border: none;
      &:focus {
        outline: none;
      }
    }
  }
}
