# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-04-04 21:09
from __future__ import unicode_literals

import django.db.models.deletion
import django.utils.timezone
import jsonfield.fields
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0024_auto_20160325_1916'),
    ]

    operations = [
        migrations.CreateModel(
            name='Iteration',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('start_datetime', models.DateTimeField(
                    default=django.utils.timezone.now)),
                ('end_datetime', models.DateTimeField(blank=True, null=True)),
                ('status', models.IntegerField(choices=[
                 (0, 'Processing'), (1, 'Requested Review'), (2, 'Provided Review')])),
                ('submitted_data', jsonfield.fields.JSONField(blank=True, default={})),
                ('assignment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                 related_name='iterations', to='orchestra.TaskAssignment')),
            ],
        ),
    ]
