# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-29 16:42
from __future__ import unicode_literals

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0052_auto_20160428_2142'),
    ]

    operations = [
        migrations.AddField(
            model_name='worker',
            name='slack_user_id',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]
