# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-01-21 15:41
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import jsonfield.fields
import orchestra.models.core.mixins
import orchestra.utils.models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0074_auto_20180116_1523'),
    ]

    operations = [
        migrations.CreateModel(
            name='SanityCheck',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('is_deleted', models.BooleanField(default=False)),
                ('handled_at', models.DateTimeField(blank=True, null=True)),
                ('check_slug', models.CharField(max_length=200)),
                ('project', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='sanity_checks', to='orchestra.Project')),
            ],
            bases=(orchestra.models.core.mixins.SanityCheckMixin, orchestra.utils.models.DeleteMixin, models.Model),
        ),
        migrations.AddField(
            model_name='workflowversion',
            name='sanity_checks',
            field=jsonfield.fields.JSONField(default={}),
        ),
    ]
